;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;Wind Chimes by Greg Harder, 2024
;================================

ORG 4084

@Seed=4032			;Random Number Seed
@LastK=4025			;16421, IY+32, Keypress status
@Delay=403C			;16444, Delay
@Freq=4040			;16448 to 16462 Freq Table

@Chimes
;================================
	LD A,(LastK)		;Test for key press
	CP FF
	JR NZ Chimes		;Loop if key pressed
	LD A,(Delay)		;POKE 16444,Delay
	LD (Ring+2),A		;Set Delay
@ChimeLup
	LD A,(LastK)		;Test for key press
	CP FF
	RET NZ			;Return to BASIC if key preeed
				;Set 3 random chimes
	CALL RND8Bit		;A=1 to 6
	LD (ChimeA+1),A		;Set random chime number
	CALL RND8Bit		;A=1 to 6
	LD (ChimeB+1),A		;Set random chime number
	CALL RND8Bit		;A=1 to 6
	LD (ChimeC+1),A		;Set random chime number
				;Start the chimes
@ChimeA
	LD A,00			;ChimeA random value
	LD B,01			;RX=Channel A coarse tone
	CALL BangAGong		;Sound ChimeA
@ChimeB
	LD A,00			;ChimeB random value
	LD B,03			;RX=Channel B coarse tone
	CALL BangAGong		;Sound ChimeB
@ChimeC
	LD A,00			;ChimeC random value
	LD B,05			;RX=Channel C coarse tone
	CALL BangAGong		;Sound ChimeC
	JR ChimeLup		;Loop until key pressed
;===============================
@BangAGong
;===============================
	LD HL,Freq		;Base address of freq. table
	LD D,00			;Locate freq. in table
	LD E,A			;Get A,B or C chime
	AND A			;Clear CY
	RL E			;Double it
	ADD HL,DE		;HL=Freq. address
				;RX=Channel A,B or C coarse tone
	LD C,(HL)		;DX=Freq. coarse
	CALL Send2AY		;Set coarse tone
	DEC B			;RX=Channel A,B or C fine tone
	INC HL
	LD C,(HL)		;DX=Freq. fine
	CALL Send2AY		;Set fine tone
	LD B,0D			;RX=Envelope shape
	LD C,00			;DX=Envelope
	CALL Send2AY		;Sound chime
@Ring				;Pause before starting next chime
	LD BC,0001
@RingLup
	DEC BC
	LD A,B 
	OR C 
	JR NZ RingLup
	RET
;===============================
; Send to AY
;  On entry B=RX  C=DX
;===============================
@Send2AY
	LD A,B			;Set RX
	OUT DF,A		;Send RX
	LD A,C			;Set DX
	OUT 1F,A		;Send DX
	RET
;================================
; 8-Bit Random number generator.
;================================
@RND8Bit
	LD DE,(Seed)		;Get (Seed)
	LD H,E 			;H=Seed LO
	LD L,FD 		;L=253
	LD A,D 			;A=Seed HI
	OR A 			;Reset CY flag
	LD B,00 		;Clear B
	SBC HL,DE 		;Subtract seed from HL
	SBC A,B 		;Subtract CY from A
	SBC HL,DE  		;Subtract seed from HL again
	SBC A,B  		;Subtract CY from A again
	LD E,A 			;E=A
	LD D,B 			;Clear D
	SBC HL,DE		;Subtract DE from HL 
	JR NC +1		;If HL positive, set random number now
	INC HL 			;Otherwise increase HL first
	LD (Seed),HL		;Set (Seed) value
@Mod
	LD A,6			;Upper limit
	LD C,A			;Set BC to dividend
	LD B,00			;Limit Range, 0 to 255
	LD H,B
@RandLoop
	OR A			;Clear CY
	SBC HL,BC		;Subtract dividend from HL
	JP P RandLoop		;Keep doing it till negative
	ADD HL,BC		;Add the divided back again
	LD A,L
	INC A			;Range=1 to Upper limit
	RET
